#include "non_gui_defs.h"

#include "interface.h"
#include "support.h"
#include "callbacks.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "gui_defs.h"
#include "gui_statistics.h"

/* write a dog or kennel statistics page to dog_info.
   the sorting has already been done previously.
   type = 10: show kennel statistics.
   type = 20: take into account all dogs
   type 0 - 5: take into account only dogs on corresponding levels.
   there are two pages of statistics. for dogs: one with name, kennel,
   fastest times in season, points, wins and number of races; the other
   one with name, kennel, fastest times ever, points/race, wins/race
   and number of races.
   for kennels: the first page just like for dogs; the second one
   with average weighted time points/race, wins/race and number of
   races */
gint write_statistics(GtkWidget *widget, gint type,
		      gint attribute)
{
    GtkWidget *dog_info = lookup_widget(widget, "dog_info");
    gint i;
    gint page = (attribute < 61) ? 1 : 2;
    gint statistics_array;
    gint dog_info_width[8] = {113, 112, 32, 32, 32, 24, 24, 18};
    gchar dog_info_title[8][16];
    gchar temporary_char[10];

    gtk_clist_freeze(GTK_CLIST(dog_info));
    gtk_clist_clear(GTK_CLIST(dog_info));
        
    if(type != 10)
    {	
	strcpy(dog_info_title[0], "(L) Name");
	strcpy(dog_info_title[1], "Kennel");
    }
    else
    {
	strcpy(dog_info_title[0], "Kennel");
	strcpy(dog_info_title[1], "Dog");
    }

    if(page == 1)
    {
	statistics_array = 0;
	for(i=0;i<3;i++)
	{
	    strcpy(dog_info_title[i + 2], "BTS ");
	    sprintf(temporary_char, "%d", i + 1);
	    strcat(dog_info_title[i + 2], temporary_char);
	}
	
	strcpy(dog_info_title[5], "Pts");
	strcpy(dog_info_title[6], "Wins");
	strcpy(dog_info_title[7], "R");   
    }
    else
    {
	statistics_array = 1;
	if(type != 10)
	{
	    for(i=0;i<3;i++)
	    {
		strcpy(dog_info_title[i + 2], "BTE ");
		sprintf(temporary_char, "%d", i + 1);
		strcat(dog_info_title[i + 2], temporary_char);
	    }

	    strcpy(dog_info_title[5], "P/R");
	    strcpy(dog_info_title[6], "W/R");
	    strcpy(dog_info_title[7], "R");    
	}
	else
	{
	    strcpy(dog_info_title[2], "WAT");
	    strcpy(dog_info_title[3], "P/R");
	    strcpy(dog_info_title[4], "W/R");
	    strcpy(dog_info_title[5], "R");
	    strcpy(dog_info_title[6], "");
	    strcpy(dog_info_title[7], "");
	}	
    }

    format_dog_info(widget, dog_info_width, dog_info_title,
		    type, page, 0, 50);

    write_statistics_list(widget, type,
			  statistics_order[statistics_array], attribute);
    
    gtk_clist_thaw(GTK_CLIST(dog_info));

    return 0;
}

/* sort all dogs by an attribute.
   attribute = 20: fastest sprgint in current season
   attribute = 30: fastest 2-lap race in current season
   attribute = 40: fastest 3-lap race in current season
   attribute = 50: points
   attribute = 60: wins
   attribute = 70: fastest sprgint ever
   attribute = 80: fastest 2-lap race ever
   attribute = 90: fastest 3-lap race ever
   attribute = 100: points / race
   attribute = 110: wins / race.
   time values we multiply by -1
   as we want them to be sorted ascending */
gint sort_dogs_statistics(gint attribute, gint *sort_order)
{
    gint i;
    gfloat to_be_sorted[400];

    for(i=0;i<400;i++)
    {
	if(attribute > 19 && attribute < 41)
	    to_be_sorted[i] = dogs[i].best_times[attribute / 10 - 2] * -1;
	else if(attribute == 50)
	    to_be_sorted[i] = points_of(dogs[i]);
	else if(attribute == 60)
	    to_be_sorted[i] = dogs[i].races[1];
	else if(attribute > 69 && attribute < 91)
	    to_be_sorted[i] = dogs[i].best_times[attribute / 10 - 4] * -1;

	/* with points and wins / race we only count dogs which
	   have participated in a number of races not only in one 
	   during the whole season */
	else if(attribute == 100)
	{
	    if(dogs[i].races[0] > (gfloat)month * 0.5 && dogs[i].races > 0)
		to_be_sorted[i] = (gfloat)points_of(dogs[i]) /
		    (gfloat)dogs[i].races[0];
	    else
		to_be_sorted[i] = 0;
	}
	else if(attribute == 110)
	{
	    if(dogs[i].races[0] > (gfloat)month * 0.5 && dogs[i].races > 0)
		to_be_sorted[i] = (gfloat)dogs[i].races[1] /
		    (gfloat)dogs[i].races[0];
	    else
		to_be_sorted[i] = 0;
	}

	/* push the invisible dogs to the end of the list */
	if(dogs[i].age == -1)
	{
	    if( (attribute > 19 && attribute < 41) ||
		(attribute > 69 && attribute < 91) )
		to_be_sorted[i] = -110;
	    else
		to_be_sorted[i] = -1;
	}
    }

    sort_float_array(to_be_sorted, sort_order, 0, 399);

    return 0;
}

/* sort the kennels by an attribute.
   attribute = 20: fastest sprgint in current season
   attribute = 30: fastest 2-lap race in current season
   attribute = 40: fastest 3-lap race in current season
   attribute = 50: points
   attribute = 60: wins
   attribute = 90: weighted average time
   attribute = 100: points / race
   attribute = 110: wins / race.
   time values we multiply by -1
   as we want them to be sorted ascending.

   write the 20 entries to 'sort_order' beginning
   at index 'first_index' */
gint sort_kennel_statistics(gint attribute, gint *sort_order, gint first_index)
{
    gint i;
    gfloat to_be_sorted[20];
    gint temporary_order[20];
    
    for(i=0;i<20;i++)
	if(attribute > 40 && attribute != 90)
	{
	    if(attribute < 100)
		to_be_sorted[i] = kennel_best(i, attribute);
	    else
	    {
		/* there have to be a certain number of races
		   when sorting p/r or w/r */
		if(kennel_best(i, 200) > month * 6)
		    to_be_sorted[i] = kennel_best(i, attribute);
		else
		    to_be_sorted[i] = 0;
	    }
	}
	else
	    to_be_sorted[i] = kennel_best(i, attribute) * -1;
    
    sort_float_array(to_be_sorted, temporary_order, 0, 19);

    for(i = first_index; i < first_index + 20; i++)
	sort_order[i] = temporary_order[i - first_index];

    return 0;
}

/* fill in a row with dog statistics */
gint append_dog_to_statistics_list(GtkCList *clist, dog Dog, gint page)
{
    gint i;
    gchar *row[8];
    gchar temporary_char[25];

    sprintf(temporary_char, "(%d) %s ",Dog.level % 10, Dog.name);
    namescan( &row[0], temporary_char);
    namescan( &row[1], kennels[(gint) ((gfloat)Dog.id / 20)].name);
    
    if(page == 1)
    {
	for(i=0;i<3;i++)
	    if(Dog.best_times[i] < 100)
		number_to_char(&row[i + 2], Dog.best_times[i], 6);
	    else
		namescan( &row[i + 2], "--:--");
	
	number_to_char(&row[5], (gfloat)points_of(Dog), 3);
	number_to_char(&row[6], (gfloat)Dog.races[1], 3);
    }
    else
    {
	for(i=0;i<3;i++)
	    if(Dog.best_times[i + 3] < 100)
		number_to_char(&row[i + 2], Dog.best_times[i + 3], 6);
	    else
		namescan( &row[i + 2], "--:--");

	
	if(Dog.races[0] > 0)
	{
	    number_to_char(&row[5],
			   (gfloat)points_of(Dog) / (gfloat)Dog.races[0], 6);
	    number_to_char(&row[6],
			   (gfloat)Dog.races[1] / (gfloat)Dog.races[0], 6);
	}
	else
	{
	    number_to_char(&row[5], 0, 3);
	    number_to_char(&row[6], 0, 3);
	}
    }

    number_to_char( &row[7], Dog.races[0], 3);

    gtk_clist_append(clist, row);

    return 0;
}

gint append_kennel_to_statistics_list(GtkCList *clist,
				     gint kennel_id, gint attribute)
{
    gint i, dog_id;
    gchar *row[8];
    
    namescan( &row[0], kennels[kennel_id].name );
    
    /* if the list is sorted by a fastest time,
       show the corresponding dog */
    if(attribute < 41)
    {
	dog_id = get_dog_id(kennel_id,
			    kennel_best(kennel_id, attribute));
	namescan( &row[1], dogs[dog_id].name);
    }

    else
	namescan( &row[1], "" );

    /* if this is page 1 */
    if(attribute < 61)
    {
	/* get best times */
	for(i=0;i<3;i++)
	{
	    if(kennel_best(kennel_id, (i + 2) * 10) < 100)
		number_to_char( &row[i + 2],
				kennel_best(kennel_id, (i + 2) * 10), 6);
	    else
		namescan( &row[i + 2], "--:--");	    
	}
	
	/* points */
	number_to_char( &row[5], (gint)kennel_best(kennel_id, 50), 3);
	/* wins */
	number_to_char( &row[6], (gint)kennel_best(kennel_id, 60), 3);
	/* races */
	number_to_char( &row[7], (gint)kennel_best(kennel_id, 200), 3);
    }
    /* page 2 */
    else
    {
	/* average weighted time */
	if(kennel_best(kennel_id, 90) < 45)
	    number_to_char( &row[2], kennel_best(kennel_id, 90), 6);
	else
	    namescan( &row[2], "--:--");
	/* points per race */
	number_to_char( &row[3], kennel_best(kennel_id, 100), 6);
	/* wins per race */
	number_to_char( &row[4], kennel_best(kennel_id, 110), 6);
	/* races */
	number_to_char( &row[5], (gint)kennel_best(kennel_id, 200), 3);

	namescan( &row[6], "");
	namescan( &row[7], "");
    }

    gtk_clist_append( clist, row );

    return 0;
}

/* if type is not 10, write the first 20 dogs according to
   'sort_order', 'type' (i.e. all dogs or those on a given level)
   and attribute to dog_info.
   if type is 10, write kennel statistics according to page
   to dog_info */
gint write_statistics_list(GtkWidget *widget, gint type,
			  gint *sort_order, gint attribute)
{
    GtkWidget *dog_info = lookup_widget(widget,
					"dog_info");
    gint i, dogs_written = 0;
    gint page = (attribute < 61) ? 1 : 2;

    if(type != 10)
    {
	for(i=0;i<400;i++)
	    if(type == 20 ||
	       dogs[sort_order[i]].level == type)
	    {
		append_dog_to_statistics_list(GTK_CLIST(dog_info),
					      dogs[sort_order[i]],
					      page);
		
		if(dogs[sort_order[i]].id < 20)
		    change_color_row(GTK_CLIST(dog_info),
				     dogs_written, 20);
		dogs_written++;
	    
		if(dogs_written == 20)
		    break;
	    }
    }
    else
	for(i=0;i<20;i++)
	{
	    append_kennel_to_statistics_list(GTK_CLIST(dog_info),
					     sort_order[400 + i],
					     attribute);
	    if(sort_order[400 + i] == 0)
		change_color_row(GTK_CLIST(dog_info),
				 i, 20);
	}

    return 0;
}

/* find out which page to show when the user browses the statistics
   and show it. if attribute = 0 and left_or_right is -1 or 1, the
   user has turned the page; if attribute is > 0, he has clicked on
   a column to sort the dogs.
   if both are 0, he just went to the statistics screen from somewhere.
   type = 10: show kennel statistics
   type = 20 or 0-5: show dog statistics */
gint show_statistics(GtkWidget *widget, gint attribute,
		    gint left_or_right, gint type)
{
    GtkWidget *dog_info_label_2 = lookup_widget(widget, "dog_info_label_2");
    GtkWidget *dog_info_label_4 = lookup_widget(widget, "dog_info_label_4");
    
    gchar *info_label_2;
    gint old_which_dogs;
    gint old_page;
    gint new_attribute;

    old_page = get_int_from_label(GTK_LABEL(dog_info_label_4));
    gtk_label_get(GTK_LABEL(dog_info_label_2), &info_label_2);
    old_which_dogs =
	get_int_from_label(GTK_LABEL(dog_info_label_2));

    if(attribute == 0 && type != 10)
    {	
	/* page right */
	if(left_or_right == 1)
	{
	    if(old_page == 1)
	    {
		new_attribute = 70;
		if(strcmp(info_label_2, " all ") == 0)
		    type = 20;
		else
		    type = old_which_dogs;
	    }
	    else
	    {
		new_attribute = 60;
		if(strcmp(info_label_2, " all ") == 0)
		    type = 5;
		else
		{
		    if(old_which_dogs > 0)
			type = old_which_dogs - 1;
		    else
			type = 20;
		}
	    }
	}

	/* page left */
	else if(left_or_right == -1)
	{
	    if(old_page == 2)
	    {
		new_attribute = 60;
		if(strcmp(info_label_2, " all ") == 0)
		    type = 20;
		else
		    type = old_which_dogs;
	    }
	    else
	    {
		new_attribute = 70;
		if(strcmp(info_label_2, " all ") == 0)
		    type = 0;
		else
		{
		    if(old_which_dogs < 5)
			type = old_which_dogs + 1;
		    else
			type = 20;
		}
	    }
	}
	else
	{
	    type = 20;
	    new_attribute = 20;
	}
    }
    
    /* sort by new attribute */
    else if(attribute != 0 && type != 10)
    {
	/* adjust attribute; from the callback function we only
	   get attributes 20 - 60 */
	attribute = attribute + (old_page - 1) * 50;
	
	sort_dogs_statistics(attribute, statistics_order[old_page -  1]);
	
	new_attribute = attribute;
	if(strcmp(info_label_2, " all ") == 0)
	    type = 20;
	else
	    type = old_which_dogs;
    }
    
    else if(type == 10)
    {
	type = 10;
	if( left_or_right != 0 )
	{
	    if(old_page == 1)
		new_attribute = 90;
	    else
		new_attribute = 20;
	}
	else
	{
	    if(attribute != 0)
	    {
		new_attribute = attribute +
		    (old_page - 1) * 70;

		sort_kennel_statistics(new_attribute,
				       statistics_order[old_page - 1], 400);
	    }
	    else
		new_attribute = 20;
	}
    }
    
    write_statistics(widget, type, new_attribute);
    
    return 0;
}

/* display the current standings in the champions' race
   (i.e., the first 6 kennels in the categories best WAT,
   most points / race and most wins / race) */
gint show_champions_race(GtkWidget *widget)
{
    GtkWidget *dog_info = lookup_widget(widget, "dog_info");
    gint i, j;
    gint dog_info_width[8] = {20, 200, 140, 0, 0, 0, 0, 0};
    gchar dog_info_title[8][16];
    gint best_kennels[3][6];
    gint temporary_order[20];
    
    gtk_clist_freeze(GTK_CLIST(dog_info));

    strcpy(dog_info_title[0], "");
    strcpy(dog_info_title[1], "Kennel");
    strcpy(dog_info_title[2], "Value");
    for(i=3;i<8;i++)
	strcpy(dog_info_title[i], "");
    
    format_dog_info(widget, dog_info_width, dog_info_title,
		    0, 0, 0, 60);

    for(i=0;i<3;i++)
    {
	sort_kennel_statistics(i * 10 + 90, temporary_order, 0);
	for(j=0;j<6;j++)
	    best_kennels[i][j] = temporary_order[j];
    }
        
    write_qualified_kennels();

    write_champions_race(widget, best_kennels);
    
    gtk_clist_thaw(GTK_CLIST(dog_info));

    return 0;
}

/* write the best 6 kennels in the categories 'WAT',
   p/r and w/r to dog_info and mark the qualified kennels
   with color */
gint write_champions_race(GtkWidget *widget, gint best_kennels[][6])
{
    GtkWidget *dog_info = lookup_widget(widget, "dog_info");
    gint i, j;
    gchar *row[8];
    
    gtk_clist_clear(GTK_CLIST(dog_info));

    namescan( &row[0], "");
    namescan( &row[1], "BEST WEIGHTED AVERAGE TIME");
    for(i=2;i<8;i++)
	namescan( &row[i], "");
    
    gtk_clist_append(GTK_CLIST(dog_info), row);

    for(i=0;i<3;i++)
    {
	for(j=0;j<6;j++)
	{
	    number_to_char( &row[0], j + 1, 3);
	    namescan( &row[1], kennels[ best_kennels[i][j] ].name );
	    if(kennel_best(best_kennels[i][j], 90 + i * 10) < 100)
		number_to_char( &row[2], kennel_best(best_kennels[i][j],
						     90 + i * 10), 6);
	    else
		namescan( &row[2], "--:--");

	    gtk_clist_append(GTK_CLIST(dog_info), row);
	    
	    if(is_in_list(qualified_kennels, 0, 5, best_kennels[i][j]) == 1)
		change_color_row(GTK_CLIST(dog_info),
				 clist_last_row(GTK_CLIST(dog_info)), 2);
	}
	
	for(j=0;j<8;j++)
	    namescan( &row[j], "");
	gtk_clist_append(GTK_CLIST(dog_info), row);
	
	if(i == 0)
	{
	    namescan( &row[0], "");
	    namescan( &row[1], "MOST POINTS PER RACE");
	    for(j=2;j<8;j++)
		namescan( &row[j], "");
	    gtk_clist_append(GTK_CLIST(dog_info), row);
	}
	else if(i == 1)
	{
	    namescan( &row[0], "");
	    namescan( &row[1], "MOST WINS PER RACE");
	    for(j=2;j<8;j++)
		namescan( &row[j], "");
	    gtk_clist_append(GTK_CLIST(dog_info), row);
	}
    }

    return 0;
}

/* show the records in 'dog_info' */
gint show_all_time_records(GtkWidget *widget)
{
    GtkWidget *dog_info = lookup_widget(widget, "dog_info");
    gint i, rows_written = 1;
    gint dog_info_width[8] = {120, 35, 200, 50, 0, 0, 0, 0};
    gchar dog_info_title[8][16];
    gchar *row[8];
    gchar temporary_char[50];

    strcpy(dog_info_title[0], "Category");
    strcpy(dog_info_title[1], "Value");
    strcpy(dog_info_title[2], "Name (Kennel)");
    strcpy(dog_info_title[3], "Season");

    for(i=4;i<8;i++)
	strcpy(dog_info_title[i], "");

    gtk_clist_freeze(GTK_CLIST(dog_info));

    format_dog_info(widget, dog_info_width, dog_info_title,
		    0, 0, 0, 70);

    gtk_clist_clear(GTK_CLIST(dog_info));

    namescan( &row[0], "DOG RECORDS");
    for(i=1;i<8;i++)
	namescan( &row[i], "");
    gtk_clist_append(GTK_CLIST(dog_info), row); g_free(row[0]);

    namescan( &row[0], "");
    gtk_clist_append(GTK_CLIST(dog_info), row); g_free(row[0]);

    for(i=0;i<7;i++)
	if( (dog_records[i].value < 100 && i < 3) ||
	    (dog_records[i].value > 0 && i > 2) )
	{
	    if(i == 0)
		namescan( &row[0], "Fastest 1-lap race:");
	    else if(i == 1)
		namescan( &row[0], "Fastest 2-lap race:");
	    else if(i == 2)
		namescan( &row[0], "Fastest 3-lap race:");
	    else if(i == 3)
		namescan( &row[0], "Most points:");
	    else if(i == 4)
		namescan( &row[0], "Most points / race:");
	    else if(i == 5)
		namescan( &row[0], "Most wins:");
	    else if(i == 6)
		namescan( &row[0], "Best winning rate:");

	    if( i != 5 && i != 3)
		number_to_char( &row[1], dog_records[i].value, 6);
	    else
		number_to_char( &row[1], dog_records[i].value, 3);

	    sprintf(temporary_char, "%s (%s)", dog_records[i].name,
		    dog_records[i].kennel_name);
	    namescan( &row[2], temporary_char);

	    number_to_char( &row[3], dog_records[i].season, 3);

	    gtk_clist_append(GTK_CLIST(dog_info), row);
	    rows_written++;
	    g_free(row[0]); g_free(row[1]);
	    g_free(row[2]); g_free(row[3]);

	    if(strcmp(dog_records[i].kennel_name, kennels[0].name) == 0)
		change_color_row(GTK_CLIST(dog_info),
				 rows_written, 20);
	}

    for(i=0;i<4;i++)
	namescan( &row[i], "");
    gtk_clist_append(GTK_CLIST(dog_info), row); g_free(row[0]);
    
    
    namescan( &row[0], "KENNEL RECORDS");
    gtk_clist_append(GTK_CLIST(dog_info), row); g_free(row[0]);

    namescan( &row[0], "");
    gtk_clist_append(GTK_CLIST(dog_info), row); g_free(row[0]);

    rows_written += 3;

    for(i=0;i<5;i++)
    {
	if( (i == 0 && kennel_records[i].value < 100) ||
	    (i > 0 && kennel_records[i].value > 0) )
	{
	    if(i == 0)
		namescan( &row[0], "Fastest WAT time:");
	    else if(i == 1)
		namescan( &row[0], "Most points:");
	    else if(i == 2)
		namescan( &row[0], "Most points / race:");
	    else if(i == 3)
		namescan( &row[0], "Most wins:");
	    else if(i == 4)
		namescan( &row[0], "Best winning rate:");

	    if(i != 1 && i != 3)
		number_to_char( &row[1], kennel_records[i].value, 6);
	    else
		number_to_char( &row[1], kennel_records[i].value, 3);

	    namescan( &row[2], kennel_records[i].name);

	    number_to_char( &row[3], kennel_records[i].season, 3);

	    gtk_clist_append(GTK_CLIST(dog_info), row);
	    rows_written++;
	    g_free(row[0]); g_free(row[1]);
	    g_free(row[2]); g_free(row[3]);

	    if(strcmp(kennel_records[i].name, kennels[0].name) == 0)
		change_color_row(GTK_CLIST(dog_info),
				 rows_written, 20);
	}
    }
    
    gtk_clist_thaw(GTK_CLIST(dog_info));

    return 0;
}


/* fill a new history element.
   dog_history = 0: allocate memory and fill in
   trophy-winning kennels (except for the champion); called the
   semi-month before the trophy races
   dog_history = 1: fill in the rest; called in the first week
   of the new season */
gint write_new_history(gint dog_history)
{
    gint i;
    history_element *new_element = first_history;

    if(dog_history == 0)
    {
	if(first_history == NULL)
	{
	    new_element = (history_element*)xmalloc(sizeof(history_element));
	    new_element->next = NULL;
	    first_history = new_element;
	}
	else
	{	
	    while(new_element->next != NULL)
		new_element = new_element->next;

	    new_element->next =
		(history_element*)xmalloc(sizeof(history_element));
	    new_element->next->next = NULL;
	    new_element = new_element->next;
	}

	for(i=0;i<3;i++)
	{
	    write_history_kennel(&new_element, (i + 9) * 10);
	    write_history_user_kennel(&new_element, (i + 9) * 10);
	}

    }
    
    else
    {
	/* set month to 12 again and decrement season so as
	   not to get a defective sorting order in 'sort_dogs'
	   and a false season in 'write_dog_record' */
	season--;
	month = 12;
	
	while(new_element->next != NULL)
	    new_element = new_element->next;
	
	/* write general dog history */
	/* best times, most points, most wins */
	for(i=0;i<5;i++)
	    write_history_dog(new_element->dog_names, 
			      new_element->dog_kennel_names, i,
			      &(new_element->dog_values[i]), (i + 2) * 10);
	/* best p/r */
	write_history_dog(new_element->dog_names, 
			  new_element->dog_kennel_names, 5,
			  &(new_element->dog_values[5]), 100);
	/* best w/r */
	write_history_dog(new_element->dog_names, 
			  new_element->dog_kennel_names, 6,
			  &(new_element->dog_values[6]), 110);
	
	/* write user dogs' times */
	for(i=0;i<3;i++)
	    write_history_dog(new_element->user_dog_names, NULL, i,
			      &(new_element->user_dog_values[i]),
			      (i + 2) * 10);

	/* find the champions' trophy winner */
	write_history_champion(&new_element);

	/* reset month and season */
	month = 1;
	season++;
    }

    return 0;
}

/* fill in a dog in a history element according to the
   category determined by 'attribute'. 'attribute' is the
   same as in sort_dogs_statistics'.
   if there's a new dog record in the p/r or w/r categories,
   write it down */
gint write_history_dog(gchar dog_names[][19], gchar kennel_names[][14],
		      gint index, gfloat *value, gint attribute)
{
    gint temporary_order[400];
    gint i, idx = 0;

    sort_dogs_statistics(attribute, temporary_order);

    if(kennel_names != NULL)
    {
	strcpy(kennel_names[index],
	       kennels[(gint)((gfloat)dogs[temporary_order[0]].id / 20)].
	       name);
    }
    else
    {
	while(dogs[temporary_order[idx]].id > 19 ||
	      dogs[temporary_order[idx]].age == -1)
	    idx++;
    }
	
    strcpy(dog_names[index], dogs[temporary_order[idx]].name);
    for(i=0;i<3;i++)
	if(attribute == (i + 2) * 10)
	    *value = dogs[temporary_order[idx]].best_times[i];	    
    
    if(attribute == 50)
    {
	*value = points_of(dogs[temporary_order[idx]]);
	if(*value > dog_records[3].value)
	    write_dog_record(&dog_records[3],
			     temporary_order[idx], 50);
    }
    else if(attribute == 60)
    {
	*value = dogs[temporary_order[idx]].races[1];
	    if(*value > dog_records[5].value)
		write_dog_record(&dog_records[5],
				 temporary_order[idx], 60);
    }
    else if(attribute == 100)
    {
	if(dogs[temporary_order[idx]].races[0] > 0)
	{
	    *value = (gfloat)points_of(dogs[temporary_order[idx]]) /
		(gfloat)dogs[temporary_order[idx]].races[0];
	    if(*value > dog_records[4].value)
		write_dog_record(&dog_records[4],
				 temporary_order[idx], 100);
	}
	else
	    *value = 0;
    }
    else if(attribute == 110)
    {
	if(dogs[temporary_order[idx]].races[0] > 0)
	{
	    *value = (gfloat)dogs[temporary_order[idx]].races[1] /
		(gfloat)dogs[temporary_order[idx]].races[0];
	    if(*value > dog_records[6].value)
		write_dog_record(&dog_records[6],
				 temporary_order[idx], 110);
	}
	else
	    *value = 0;
    }
    
    return 0;
}

gint write_history_kennel(history_element **element, gint attribute)
{
    gint index = attribute / 10 - 8;
    gint record_index = attribute / 5 - 18;
    gint temporary_order[20];

    sort_kennel_statistics(attribute, temporary_order, 0);
    
    strcpy((*element)->kennel_names[index],
	   kennels[temporary_order[0]].name);
    (*element)->kennel_values[index] =
	kennel_best(temporary_order[0], attribute);

    /* check whether there's a new p/r or w/r kennel
       record */    
    if(attribute > 99 &&
       kennel_best(temporary_order[0], attribute) >
       kennel_records[record_index].value)
	write_kennel_record(&kennel_records[record_index],
			    temporary_order[0],
			    kennel_best(temporary_order[0], attribute));
    
    return 0;
}

/* find out the ranking of the human player in the three
   kennel trophy categories */
gint write_history_user_kennel(history_element **element, gint attribute)
{
    gint i;
    gint index = (attribute / 10) - 8;
    gint temporary_order[20];
    
    sort_kennel_statistics(attribute, temporary_order, 0);
    
    for(i=0;i<20;i++)
    {
	if(temporary_order[i] == 0)
	{
	    (*element)->user_kennel_results[index] = i + 1;
	    (*element)->user_kennel_values[index] =
		kennel_best(0, attribute);
	}	
    }

    return 0;
}

/* find the champions' trophy winner and note the user's performance in
   the trophy races (or non-performance), too*/
gint write_history_champion(history_element **element)
{
    gint i,j;
    gint temporary_order[6];
    gint kennels_participating[6] = {-1,-1,-1,-1,-1,-1};
    gfloat to_be_sorted[6];
    gint winning_points;
    gint user_takes_part = 0;
    
    /* get total points at the end of the last race */
    for(j=0;j<6;j++)
	for(i=0;i<6;i++)
	{
	    if(races[0][j].dogs[i] != NULL)
	    {
		kennels_participating[i] =
		    (gint)((gfloat)races[0][j].dogs[i]->id / 20);

		to_be_sorted[i] =
		    (gfloat)trophy_points_of_kennel(kennels_participating[i], 5);

		if(kennels_participating[i] == 0)
		{
		    (*element)->user_kennel_values[0]
			= to_be_sorted[i];
		    user_takes_part = 1;
		}
	    }
	    /* if the human player hasn't entered a dog */
	    else
	    {
		kennels_participating[i] = 0;
		to_be_sorted[i] =
		    (gfloat)trophy_points_of_kennel(0, 5);

		(*element)->user_kennel_values[0]
		    = to_be_sorted[i];
		user_takes_part = 1;
	    }
	}

    sort_float_array(to_be_sorted, temporary_order, 0, 5);

    for(i=0;i<6;i++)
	if(kennels_participating[temporary_order[i]] == 0)
	    (*element)->user_kennel_results[0] = i + 1;

    /* if the human player wasn't qualified, we note it */
    if(user_takes_part == 0)
    {
	(*element)->user_kennel_values[0] = 0;
	(*element)->user_kennel_results[0] = -1;
    }

    /* now if there is more than one kennel in first,
       we sort those who have the same number of points
       by WAT over all six races.
       the rest remains on the same ranks as before */
    winning_points =
	trophy_points_of_kennel(kennels_participating[temporary_order[0]], 5);
    
    (*element)->kennel_values[0] = to_be_sorted[temporary_order[0]];

    if(trophy_points_of_kennel(
	   kennels_participating[temporary_order[1]], 5) < winning_points)
    {
	strcpy((*element)->kennel_names[0],
	       kennels[kennels_participating[temporary_order[0]]].name);
	return 1;
    }
    else
    {

	for(i=0;i<6;i++)
	    if(trophy_points_of_kennel(
		   kennels_participating[temporary_order[i]], 5) ==
	       winning_points)
	    {
		to_be_sorted[temporary_order[i]] = 
		    trophy_wat(kennels_participating[temporary_order[i]]) * -1;
	    }
	    else 
		to_be_sorted[temporary_order[i]] = -100;
    
	sort_float_array(to_be_sorted, temporary_order, 0, 5);

	strcpy((*element)->kennel_names[0],
	       kennels[kennels_participating[temporary_order[0]]].name);
	
	/* if the user's kennel was tied for first,
	   note the (maybe new) result */
	for(i=0;i<6;i++)
	    if(kennels_participating[temporary_order[i]] == 0 &&
	       trophy_points_of_kennel(0,5) == winning_points)
		(*element)->user_kennel_results[0] = i + 1;
    }
    
    return 0;
}

/* compute the WAT for a kennel from the 6 trophy races */
gfloat trophy_wat(gint kennel_id)
{
    gint i,j;
    gfloat sum = 0;

    for(i=0;i<6;i++)
    {
	for(j=0;j<6;j++)
	{
	    if(races[0][i].dogs[j] != NULL)
	    {
		if((gint)((gfloat)races[0][i].dogs[j]->id / 20) == kennel_id)
		{
		    if(i < 2)
			sum += races[0][i].times[j] * 2;
		    else if(i < 4)
			sum += races[0][i].times[j];
		    else
			sum += races[0][i].times[j] * 0.6666666;
		}
	    }
	}
    }

	return sum / 6;
}
