/* gui headers */
#include <stdio.h>
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Structures ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

typedef struct _dog dog;
struct _dog
{
    gchar name[19];

    /* each dog in the game has an identity number */
    gint id;

    /* parents' ids and names; needed later to prevent (or perform)
       incest. we take not only the id's because dogs older than 5
       years retire from cpu-kennels but their id's remain in the game */
    gchar mother_name[19];
    gchar father_name[19];

    gint mother_id;
    gint father_id;
    
    /* 0 dog 1 bitch */
    gint gender;

    /* level between 0 and 5 expressing the dog's racing skill.
       successes in races and good times lead to advancement */
    gint level;

    /* age is measured in months */
    gfloat age;

    /* 0 < status < 20: normal; if it's a bitch, months until
       heat is 'status';
       status = 0: bitch is in heat
       19 < status < 26: injury; duration of injury is status - 19
       X099 < status < X499: bitch is pregnant, birth in X semi-months;
       father is dog with id status % 1000 - 100
       thus we know that a bitch with 17 < status < 499 bears.
       if status = 18 for a dog, we know he'll be the father of puppies and
       we don't remove him from the game even if he's old */
    gint status;
        
    /* these two arrays consisting of 0's and 1's are the genetic code
       of the dog and are used together with the ancestor coefficients
       when interbreeding is done.

       they don't give the actual skill of the dog but rather the potential:
       the actual value is computed by multiplying them by the fitness value.

       the speed and stamina values the human player sees are computed by
       summing up the arrays, which means they are between 0 and 99.

       high speed and high stamina are in most of the cases mutually exclusive,
       i.e. dogs tend to prefer either a short race or a long one */
    gint speed[99]; 
    gint stamina[99];

    /* these arrays describe the genetic heritage of the dog
       and play a role when interbreeding */
    gint ancestor_speed[99];
    gint ancestor_stamina[99];

    /* a number between 0 and 1 which determines how much of speed
       and stamina are actually 'available'.
       a young dog's fitness increases gradually until reaching a peak
       approximately at the age of three and decreases afterwards. */
    gfloat fitness;

    /* between 0 and 1, determines the chance of
       successful interbreeding */
    gfloat fertility;

    /* the best times in a one, two or three-lap race during the current season
       (first three entries) and all-time records (last three entries) */
    gfloat best_times[6];

    /* the number of races, wins, 2nd - 5th places, respectively */
    gint races[6];

    /* the last five ranks and times */
    gfloat last_races[10];
};

typedef struct _kennel kennel;
struct _kennel
{
    gchar name[14];
    
};

/* in such a struct we store at the end of a season
   various information about the season */
typedef struct _history_element history_element;
struct _history_element
{
    /* the champion kennel and the three
       trophy winners */
    gchar kennel_names[4][14];
    /* the corresponding values (best 'WAT' etc.) */
    gfloat kennel_values[4];

    /* the dogs with the fastest times for each race
       distance, best winning and p/r rate and most
       wins and points */
    gchar dog_names[7][19];
    gchar dog_kennel_names[7][14];
    gfloat dog_values[7];
    
    /* the user's dogs' fastest times */
    gchar user_dog_names[3][19];
    gfloat user_dog_values[3];

    /* the user's kennels performance in the
       kennel categories */
    gint user_kennel_results[4];
    gfloat user_kennel_values[4];

    /* we will store the histories in a chained list;
       at the end of each season, we allocate memory for
       another element and fill it in */
    history_element *next;
};

typedef struct _race race;
struct _race
{
    gint level;
    gint laps;

    dog *dogs[10];

    /* results */
    gfloat times[10];

    /* indices of the dogs in the order
       they finished.
       e.g. race.dogs[ ranks[0] ] is first */
    gint ranks[10];
};

typedef struct _dog_record dog_record;
struct _dog_record
{
    gchar name[19];
    gchar kennel_name[14];
    gint season;
    gfloat value;    
};

typedef struct _kennel_record kennel_record;
struct _kennel_record
{
    gchar name[14];
    gint season;
    gfloat value;    
};


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Global variables ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* all dogs in the game */
/* 400 to 419 are the human player's puppies */
dog dogs[420];

/* all kennels in the game */
kennel kennels[20];

/* the six kennels qualified for the champions'
   trophy races */
gint qualified_kennels[6];

/* the order in which the dogs are shown to the human player 
   plus type of ordering stored in 420.
   example: the dogs from 0 to 19 are shown to the human player
   in the order dogs[ order[0] ] to dogs[ order[19] ]*/
gint order[421];

/* the order in which the dogs in the dog statistics window
   are shown. as there are two pages of statistics we have two
   arrays */
gint statistics_order[2][420];

/* season, month and week numbers.
   week alternates between 1 and 3 */
gint season, month, week;

/* a buffer of names kept in memory to name each new dog in the game */
gchar names_kept[500][19];

/* the results of last 'semi-month''s races ([0]) and races of
   current semi-month */
race races[2][12];

/* a the first history element  */
history_element *first_history;

/* all-time records */

/* entries 0 - 2: fastest times over 1, 2 or 3 laps
   3, 4: most points, most points / race
   5, 6: most wins, best winning rate  */
dog_record dog_records[7];

/* entry 0: weighted average time
   1, 2: most points, most points / race
   3, 4: most wins, best winning rate  */
kennel_record kennel_records[5];

/* the name of the file under which the game
   has been most recently saved */
gchar *game_file;

/* some options */

/* if the human player tries too hard to force a cpu-owner
   to trade or interbreed dogs, the cpu-owner refuses to deal
   for some weeks.
   options_kennel_block[0][i] is the value for kennel i + 1 for
   interbreeding dogs, options_kennel_block[1][i] for trading dogs */
gint options_kennel_block[2][19];

/* confirmation options. default value is 1, so that
   the user is always asked to confirm things like giving
   away dogs etc.
   entries:
   0: giving away dogs
   1: go to races
   2: disadvantageous mating
   3: incest
   4: quit when not saved*/
gint options_confirm[12];

/* whether to show news
   0: new puppies born
   1: no room for new puppies on puppy list
   2: puppy promotion warning
   3: puppies given away
   4: old dogs given away
   5: fighting in the kennel
   6: awards at the end of the season */
gint options_show_news[12];

/* 0: whether or not to show the 'racing...' progress
   bar or live races before showing the race results

   1: whether or not to pre-select races and
   dogs when entering them

   2: whether or not to show all races
   (and not only those the human player's dogs
   participate in)

   3: whether the 'save' button overwrites the current
   greyhounds file or pops up the 'save as' file selection

   4: whether or not some white spaces are prepended when
   printing a message in the message window; used only
   in the print_message function; it simply makes it easier
   for the user to see that a message HAS been printed.
   this is actually an internal variable, not a real option

   5 - 10: whether or not the live races on levels 0 to 5
   are shown 

   17: the type of conjunction between 18 and 19: 1 means
   OR, i.e. we notify if there's a bitch with either the 
   specified speed or the specified stamina; 0 means we
   notify if there's a bitch who surpasses both values

   18: the speed value limit; we notify the
   user if there's a bitch in heat surpassing it;
   default value is 199 (-> no notification)

   19: the stamina value */
gint options_misc[50];



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Functions from 'math_functions.c' ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* generate a random number from the uniform distribution between a and b */
double rnd(double a,double b);

/* generate a random number from the gauss distribution */
double gaussrand(void);

/* generate a random number from the gauss distribution N((b+c)/2, s^2)
   that's guaranteed to be between a and d and with probability 99,7%
   is between b and c */
double gaussvt(double a,double b,double c,double d);

/* write a permutation of the integers between 'min' and 'max'
   into the array 'perm' */
gint write_permutation(gint *perm, gint min,gint max);

/* sum up the elements of an integer array between indices
   'min' and 'max' */
gfloat sum(gint *array, gint min, gint max);

/* compute the smallast element of an array from element
   first to element last */
gint min_int(gint *array, gint first, gint last);
gfloat min_float(gfloat *array, gint first, gint last);



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Functions from 'misc.c' ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* avoid segfaults with 'malloc' */
void *xmalloc(size_t size);

/* check whether 'item' is in array 'list' between
   'min' and 'max' */
gint is_in_list(gint *list, gint min, gint max, gint item);

/* read an input up to 'maxlen' characters from the terminal 
   into the gchar array 'q' */
gint getinput(gchar *q,gint maxlen);

/* avoid segfaults that occur with 'fopen' by using
   ffopen; additionally the user can specify here the path
   of the needed text files */
FILE *ffopen(const gchar *file, const gchar *mode);

/*  transform a value between 0 and 9.9 into an array
    of 0's and 1's that signifies part of the genetic
    code of a dog */
gint generate_code(gint *array, gfloat value);

/* allocate the memory needed
   for 'dest' and copy 'source' into it */
gint namescan(gchar **dest, gchar *source);

/* sort a gfloat array between indices 'first_element' and 'last_element';
   the function assumes that 'order' can hold
   'last_element' - 'first_element' + 1 entries */
gint sort_float_array(gfloat *array, gint *order, 
		     gint first_element, gint last_element);

/* allocate memory for 'dest' depending on 'size' and write the
   number 'number' into it */
gint number_to_char(gchar **dest, gfloat number, gint size);

/* decompose an integer that stores multiple information
   just like the value you use whith 'chmod':
   chmod 4... means read only, 2 means write only, but
   6 = 4 + 2 means both; here we have the values 1, 2, 4 and 8.
   the function takes one of these and the compound value
   and returns 1 if it contains the single value and zero otherwise */
gint decompose_compound_integer(gint value, gint compound_value);

/* sleep for 'time' using 'sleep' and 'usleep' */
gint my_sleep(gfloat time);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Functions from 'hound_functions.c' ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* generate the 20 dogs for a kennel */
gint generate_batch(gchar names[][19], gint kennel_id);

/* fill in name, age, potential level, gender, fitness and race statistics.
   if type is '1' it's a randomly generated dog at startup, if it's 11
   it's a randomly generated dog replacing an old dog in a cpu-kennel;
   if it's 0 it's  a birth.

   potential level signifies the level the dog should have at the peak
   of its career, i.e. at the age of 3. the real level results from the
   skill values multiplied by the fitness value */
gint fill_in_general_data(dog *Dog, gchar names[][19], gint level, gint type);

/* generate speed etc. values at the beginning of a new game */
gint generate_skills(dog *Dog, gint level);

/* calculate a dog's level from its skill values
   at startup */
gint calculate_level_skills(dog Dog);

/* change age, fitness and (sometimes)
   level of a dog */
gint semi_monthly_reevaluation(dog *Dog);

/* return the speed or stamina value that the human player
   really sees in the game */
gfloat real_value(dog Dog, gint speed);

/* sort dogs of a kennel depending on an attribute */
gint sort_dogs(gint kennel_id, gint attribute);

/* add up the points a dog has earned based on the
   number of finishes in top 5 */
gint points_of(dog Dog);

/* return the name of the father or mother of
   a dog */
gint find_parent_of(dog Dog, gint father);

/* return the number of dogs in a kennel */
gint dogs_in_kennel(gint kennel_id);

gint kennel_has_bitch_in_heat(gint kennel_id);

/* at random dog generation make up average results
   for the last 5 races so that the monthly level
   reevaluation has something to chew on */
gint make_up_last_races(dog *Dog);

/* compare two dogs either for trading or interbreeding.
   for young dogs we take their expected peak skill, for older ones
   their current skill */
gint compare_dogs(dog Dog1, dog Dog2, gint trade);

/* swap two dogs when trading is done */
gint swap_dogs(dog *Dog1, dog *Dog2);

/* copy dog1 to dog2 */
gint copy_dogs(dog Dog1, dog *Dog2);

/* return id of dog with the given best_time */
gint get_dog_id(gint kennel_id, gfloat time);

/* compute and return the value corresponding to 
 'attribute' as described in 'sort_kennel_statistics'.
 if attribute = 200, return number of races */
gfloat kennel_best(gint kennel_id, gint attribute);

/* monthly check of the all-time records */
gint check_records(void);

/* write a new kennel record */
gint write_kennel_record(kennel_record *record, gint id, gfloat value);

/* write a new dog record */
gint write_dog_record(dog_record *record, gint id, gint type);

/* return the id of the first 'invisible' dog
   in the kennel with id 'id'; if the kennel's
   full, return -1; */
gint first_free_place_in_kennel(gint id);

/* write the six kennels that are qualified for
   the champions' trophy races  */
gint write_qualified_kennels(void);

/* sort the kennels by an attribute.
   attribute = 20: fastest sprgint in current season
   attribute = 30: fastest 2-lap race in current season
   attribute = 40: fastest 3-lap race in current season
   attribute = 50: points
   attribute = 60: wins
   attribute = 70: average weighted time
   attribute = 100: points / race
   attribute = 110: wins / race.
   time values we multiply by -1
   as we want them to be sorted ascending */
gint sort_kennel_statistics(gint attribute, gint *sort_order, gint first_index);

/* see if there was fighting in the human player's kennel */
gint fighting(void);

/* compute the points of a kennel in the trophy
   races up to race 'race_number'.
   for instance, if a kennel takes first place in the
   first trophy race and second in the second race,
   the points returned are 6 + 4 = 10 */
gint trophy_points_of_kennel(gint kennel_id, gint race_number);


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Functions from 'racing.c' ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* decide the outcome of the 12 races */
gint race_results(void);

/* select randomly the dogs for the 12 races (2 per dog level) that
   take place each month; select randomly number of laps for each race.
   the index '9' is reserved for the human player's dog */
gint write_races(void);

/* check whether a given dog is entered into one
   of the 12 races */
gint entered_in_races(dog Dog);

/* find out if any of the participating dogs has been injured.
   the probability of injury depends on fitness:
   at 0.6 it's 15% and at 1 its 0 */
gint calculate_injuries(race *Race, gint last_index);

/* write the races for the champions' trophy */
gint write_champions_trophy_races(void);

/* choose the best six dogs of a kennel and
   enter them in the six trophy races */
gint enter_kennel_in_trophy_races(gint kennel_id);

/* enter a dog in a trophy race */
gint enter_best_dog_in_race(gint kennel_id, gint race_id);



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Functions from 'breeding.c' ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* generate 1 to 6 pups using the speed and stamina codes and the
   ancestor_coefficients of the parents; 'pups' has to be an array
   of size 6. */
gint mating(dog father, dog mother, dog *pups, gint number_of_pups);

/* find out if the mating has been succesful and write the
   results into the dogs' status.
   type = 0: it's a mating with equal rights 
   type = 1: it's a disadvantageous mating for the human player */
gint mate(dog *Dog1, dog *Dog2, gint type);

/* find out whether and how two dogs are related */
gint kinship_of(dog Dog1, dog Dog2);



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Functions from 'load_save.c' ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* write all the global variables into a file */
gint save_game(const gchar *file_name);

/* read all the global variables from a file */
gint load_game(gchar *file_name, gint before_game);

/* free the history list */
int free_histories(void);
